@@ -1,6 +1,6 @@
 ##
 #
-#    Copyright 2001, AllAfrica Global Media
+#    Copyright 2001-2007, AllAfrica Global Media
 #
 #    This file is part of XML::Comma
 #
@@ -38,10 +38,12 @@ use strict;
 # _Def_indexes                : {} hashref {_Def_indexes}->{name} = $index
 # _Def_storages               : {} hashref {_Def_storages}->{name} = $storage
 # _Def_macro_names            : [] arrayref listing macros applied
-# _Def_plural                 : {} an existence hashref
-# _Def_ignore_for_hash        : {} an existence hashref
-# _Def_include_for_hash       : {} an existence hashref
-# _Def_required               : {} an existence hashref
+
+# _Def_properties             : {} an existence hashref
+#   fields: ignore_for_hash, include_for_hash, required, plural
+#   misc: nested, blob, 
+#   macro stuff: enum, boolean, range, timestamp, timestamp_created, timestamp_last_modified
+#   UI stuff: doc_key, timestamp, single_line
 #
 # _Def_sort_sub               : a code ref created by _config__sort_sub
 #
@@ -65,10 +67,8 @@ use strict;
 # from_file
 sub new {
   my ( $class, %arg ) = @_;
-  if ( $arg{file} ) {
-    return _new_from_file ( $arg{file}, $class );
-  } elsif ( $arg{block} ) {
-    return _new_from_block ( $arg{block}, $class );
+  if ( $arg{file} || $arg{block}) {
+    return _new_from_content ( %arg, top_level_class=> $class );
   }
   # called simply: this routine actually does all the work of making a
   # new def, the above ifs call it one way or another.
@@ -85,8 +85,8 @@ sub new {
 
 sub _init {
   my ( $self, %arg ) = @_;
-  $self->{_Def_plural} = {};
-  $self->{_Def_required} = {};
+  $self->{_Def_properties}->{plural} = {};
+  $self->{_Def_properties}->{required} = {};
   $self->{_Def_macro_names} = [];
   $self->{_Def_auto_escape} = 0;
   $self->{_Def_auto_unescape} = 0;
@@ -128,30 +128,20 @@ sub retrieve {
   XML::Comma::Log->err ( 'DEF_READ_ERROR', 'not allowed to retrieve a Def' );
 }
 
-sub _new_from_block {
-  my ( $block, $class ) = @_;
+# there was almost complete duplication of code between
+# _new_from_file and _new_from_block, so i combined them
+# and fixed the arguments from the caller so that we don't
+# have to differentiate.
+sub _new_from_content {
+  my %arg = @_;
   my $def = eval {
-    XML::Comma::parser()->new ( block => $block,
-                                top_level_class => $class );
+    XML::Comma::parser()->new ( %arg );
   }; if ( $@ ) {
     XML::Comma::Log->err ( 'DEF_ERROR', $@ );
   }
   return $def;
 }
 
-sub _new_from_file {
-  my ( $file, $class ) = @_;
-  my $def = eval {
-    XML::Comma::parser()->new ( filename => $file,
-                                top_level_class => $class );
-  };
-  if ( $@ ) {
-    XML::Comma::Log->err ( 'DEF_ERROR', $@ );
-  }
-  return $def;
-}
-
-
 sub finish_initial_read {
   my ( $self, $in_progress_parser ) = @_;
   $self->_Def_init_name_up_path ( $in_progress_parser );
@@ -168,6 +158,7 @@ sub finish_initial_read {
     $self->_create_storages();
   }
   $self->_config_dispatcher();
+  $self->_init_decorators();
 }
 
 
@@ -247,19 +238,43 @@ sub _create_storages {
 
 sub _index_on_store_hooks {
   my ( $self, $storage_el ) = @_;
-  foreach my $inx_name ( $storage_el->elements_group_get('index_on_store') ) {
-    # check whether the index exists, and that it expects this storage
+  
+  foreach my $index_string ( $storage_el->elements_group_get(
+                               'index_on_store') ) {
+
+    # mismatched <store/> and <doctype/> directives in an index declaration 
+    # throw an exception
+    
     my $index;
-    eval { $index = $self->get_index ( $inx_name ); };
-    if ( $@ ) {
-      die "can't index_by_name to '$inx_name' -- it doesn't exist\n";
+
+    my ( $def_name, $index_name ) = split /:/, $index_string;
+
+    if ( $def_name && $index_name ) { 
+      eval { $index = XML::Comma::Def->read( name => $def_name )
+                                     ->get_index ( $index_name ); 
+      }; if ( $@ ) {
+        die "can't index_by_name to '$index_string' -- it doesn't exist\n";
+      }
+      $self->_store_doctype_index_match_p( $index, $storage_el ) or 
+        die "can't index_by_name to '$index_string' " . 
+            "-- it doesn't accept writes from storage '" . 
+            $storage_el->name() . "' in def '" . $self->name() . "'\n";
+    } else {
+      eval { $index = $self->get_index ( $index_string ); };
+      if ( $@ ) {
+        die "can't index_by_name to '$index_string' -- it doesn't exist\n";
+      }
+      $self->_store_doctype_index_match_p( $index, $storage_el ) or
+        die "can't index_by_name to '$index_string' " . 
+            "-- it doesn't accept writes from storage '" . 
+           $storage_el->name() . "' in def '" . $self->name() . "'\n";
     }
-    my $st = $index->store();
-    if ( $st ne $storage_el->name() ) {
-      die "can't index_by_name to '$inx_name' -- it stores from '$st'\n";
-    }
-    my $update_string = "sub { \$_[0]->index_update ( index=>'$inx_name' ) };";
-    my $delete_string = "sub { \$_[0]->index_remove ( index=>'$inx_name' ) };";
+    
+    push @{$storage_el->{_index_on_stores}}, $index_string;
+    my $update_string = 
+      "sub { \$_[0]->index_update ( index=>'$index_string' ) };";
+    my $delete_string = 
+      "sub { \$_[0]->index_remove ( index=>'$index_string' ) };";
     $storage_el->add_hook ( 'post_store_hook', $update_string );
     $storage_el->add_hook ( 'erase_hook', $delete_string );
   }
@@ -278,6 +293,28 @@ sub _create_indexes {
   }
 }
 
+sub _store_doctype_index_match_p {
+  my ( $self, $index, $storage_el ) = @_; 
+  no warnings;
+
+  # check that the user defined doctype declarations match the index
+  my $doctype_match = grep { $_ eq '*' or $_ eq $self->name() }
+    map{ eval $_ } $index->elements_group_get( 'doctype' );
+
+  # or that the default index doctype is the same as the def
+  $doctype_match ||= $index->doctype() eq $self->name();
+
+  # check user defined stores declarations match the storage element
+  my $store_match = grep { $_ eq '*' or $_ eq $storage_el->name() }
+    map { eval $_ } $index->elements_group_get( 'store' );
+
+  # or that the default index store is the same as the storage element
+  $store_match ||= $index->store() eq $storage_el->name();
+
+  return $doctype_match && $store_match; 
+}
+
+
 sub get_store {
   my ( $self, $storage_name ) = @_;
   return $self->{_Def_storages}->{$storage_name} ||
@@ -346,27 +383,27 @@ sub _config__plural {
   if ( $@ ) {
     die "problem with plural list: " . $el->get() . "\n";
   }
-  map { $self->{_Def_plural}->{$_} = 1 } @plural_list;
+  map { $self->{_Def_properties}->{plural}->{$_} = 1 } @plural_list;
 }
 
 sub _config__ignore_for_hash {
   my ( $self, $el ) = @_;
   die "tried to define ignore_for_hash when we have an include_for_hash\n"
-    if($self->{_Def_include_for_hash});
+    if($self->{_Def_properties}->{include_for_hash});
   my @ignore_list = eval $el->get();
   if ( $@ ) {
     die "problem with ignore_for_hash list: " . $el->get() . "\n";
-  } map { $self->{_Def_ignore_for_hash}->{$_} = 1 } @ignore_list;
+  } map { $self->{_Def_properties}->{ignore_for_hash}->{$_} = 1 } @ignore_list;
 }
 
 sub _config__include_for_hash {
   my ( $self, $el ) = @_;
   die "tried to define include_for_hash when we have an ignore_for_hash\n"
-     if($self->{_Def_ignore_for_hash});
+     if($self->{_Def_properties}->{ignore_for_hash});
   my @include_list = eval $el->get();
   if ( $@ ) {
     die "problem with include_for_hash list: " . $el->get() . "\n";
-  } map { $self->{_Def_include_for_hash}->{$_} = 1 } @include_list;
+  } map { $self->{_Def_properties}->{include_for_hash}->{$_} = 1 } @include_list;
 }
 
 # note in the existence hashref that this is a required element, so
@@ -381,14 +418,14 @@ sub _config__required {
     die "error while trying to parse required list '$required': $@\n";
   }
   foreach my $req ( @list ) {
-    $self->{_Def_required}->{$req} = 1;
+    $self->{_Def_properties}->{required}->{$req} = 1;
     $self->add_hook ( 'validate_hook',
          "sub {
             my \$self = shift();
             my \$req_el = \$self->elements('$req')->[0];
             die \"required element '$req' not found in \" . \$self->tag_up_path() . \"\\n\"  if
                     (! \$req_el) or 
-                    ((! \$req_el->def()->is_nested()) and (! \$req_el->get()));
+                    ((! \$req_el->def()->is_nested()) and (\$req_el->get() eq ''));
           }",
                     0xffffff # our "hook order" number -- high because
                              # we want to do this hook after any that
@@ -414,6 +451,19 @@ sub _config__required {
 #      }" );
 #  }
 
+#introspection stuff - see _Def_properties, above, or Bootstrap.pm
+#for allowed fields
+sub _config__properties {
+  my ( $self, $el ) = @_;
+  foreach my $child ($el->elements()) {
+    my $tag  = $child->tag();
+    my @list = eval $child->get();
+    if ( $@ ) {
+      die "problem with properties::$tag list: " . $el->get() . "\n";
+    }
+    map { $self->{_Def_properties}->{$tag}->{$_} = 1 } @list;
+  }
+}
 
 sub _config__read_hook {
   my ( $self, $el ) = @_;
@@ -487,20 +537,20 @@ sub sort_sub {
 
 # is the element the name of which we pass plural
 sub is_plural {
-  return 1  if  $_[0]->{_Def_plural}->{$_[1]};
+  return 1  if  $_[0]->{_Def_properties}->{plural}->{$_[1]};
   return;
 }
 
 sub is_required {
-  return 1  if  $_[0]->{_Def_required}->{$_[1]};
+  return 1  if  $_[0]->{_Def_properties}->{required}->{$_[1]};
   return;
 }
 
 sub is_ignore_for_hash {
-  if($_[0]->{_Def_ignore_for_hash}) {
-    return 1  if  $_[0]->{_Def_ignore_for_hash}->{$_[1]};
-  } elsif($_[0]->{_Def_include_for_hash}) {
-    return 1  unless  $_[0]->{_Def_include_for_hash}->{$_[1]};
+  if($_[0]->{_Def_properties}->{ignore_for_hash}) {
+    return 1  if  $_[0]->{_Def_properties}->{ignore_for_hash}->{$_[1]};
+  } elsif($_[0]->{_Def_properties}->{include_for_hash}) {
+    return 1  unless  $_[0]->{_Def_properties}->{include_for_hash}->{$_[1]};
   }
   return;
 }
@@ -512,6 +562,12 @@ sub validate {
     }
 }
 
+sub has_property {
+  my ($self, $property, $el_name) = @_;
+  return 1 if $self->{_Def_properties}->{$property}->{$el_name};
+  return;
+}
+
 # is the element we define allowed to have sub-elements
 sub is_nested {
   return $_[0]->{_Def_is_nested} ||=
@@ -531,7 +587,6 @@ sub def_sub_elements {
   return ( $_[0]->elements('element','nested_element','blob_element') );
 }
 
-
 ########
 #
 # method methods -- override these from AbstractElement to point at
@@ -589,9 +644,26 @@ sub auto_dispatch {
 #
 ####
 
+sub _init_decorators {
+  my $self = shift;
+  my @classes = $self->get_decorators() or return;
+  # allow $def->decorator_method
+  if ( $self->tag eq 'DocumentDefinition' ) {
+    bless ( $self, Class::ClassDecorator::hierarchy(ref($self),@classes) );
+  }
+  foreach my $class ( @classes ) {
+    if ( $class->can('init') ) {
+      $class->init ( $self );
+    }
+  }
+}
+
 sub get_decorators {
-  return @{ $_[0]->{_Def_decorator_classes}
-              ||= [map { $_->module } $_[0]->elements('class')] };
+  no strict 'refs';
+  return @{ $_[0]->{_Def_decorator_classes} ||=
+    [map { eval "require ".$_->module  unless  scalar %{$_->module."::"};
+           die "$@\n"  if  $@;
+           $_->module } $_[0]->elements('class')] };
 }
 
 sub get_decorator_config {
