@@ -1,6 +1,6 @@
 ##
 #
-#    Copyright 2001, AllAfrica Global Media
+#    Copyright 2001-2007, AllAfrica Global Media
 #
 #    This file is part of XML::Comma
 #
@@ -84,10 +84,14 @@ sub read {
       $doc = 
         XML::Comma::Doc->new ( block=>$input_str,
                                read_args => $read_args,
-                               validate => $validate );
+                               validate => $validate,
+                               location=> $location,
+                               id=> $id,
+                               key=> $key,
+                               lock=> $lock,
+                               store_obj=> $self );
     }
     $doc->{_Doc_new} = 0; # blech, hack, yuck
-    $doc->set_storage_info ( $self, $location, $id, $key, $lock );
   }; if ( $@ ) {
     my $error = $@;
     if ( $lock ) { eval { XML::Comma->lock_singlet()->unlock($key) }; }
@@ -99,7 +103,7 @@ sub read {
 sub write {
   my ( $self, %arg ) = @_;
   # pre-store hooks
-  unless ( $arg{no_hooks} ) {
+  unless ( $arg{no_hooks} || $arg{no_pre_store_hooks} ) {
     foreach my $sub ( @{$self->get_hooks_arrayref('pre_store_hook')} ) {
       $sub->( $arg{doc}, $self, \%arg );
     }
@@ -146,6 +150,7 @@ sub write {
   my $blobs_flag = 0;
   unless ( $arg{no_blobs} ) {
     foreach my $blob ( $arg{doc}->get_all_blobs_and_ghosts() ) {
+      # print "MOVING: " . $blob->{_Blob_location} . "\n";
       $blobs_flag += $blob->store ( copy => $arg{anew} );
     }
     $arg{doc}->clear_ghosts_list();
@@ -164,8 +169,9 @@ sub write {
   # unlocked properly. we'll re-throw the first error we got after
   # we finish unlocking.
   my $post_store_error;
-  unless ( $arg{no_hooks} ) {
+  unless ( $arg{no_hooks} || $arg{no_post_store_hooks} ) {
     foreach my $sub ( @{$self->get_hooks_arrayref('post_store_hook')} ) {
+      #$sub->( $arg{doc}, $self, \%arg );
       eval { $sub->( $arg{doc}, $self, \%arg ); };
       $post_store_error = $@  if  ( $@ and ! $post_store_error );
     }
@@ -179,10 +185,14 @@ sub write {
 }
 
 # id => doc id
+# key => doc key
 # type => doc type
 # store => name of store
 # no_hooks => don't run store_hooks if true
 # doc_string => string block that is the new doc
+# blobs_local_files => flag (1) saying that we can treat this as a move between
+#              two local stores, that we'll do set_from_file for blob content,
+#              rather than get it from the blobs_hash below
 # blobs => { blob_location_from_doc_string => blob_content }
 sub force_store {
   my ( $self, %args ) = @_;
@@ -206,9 +216,20 @@ sub force_store {
     my ( $volume, $directories, $file ) = File::Spec->splitpath ( $location );
     XML::Comma::Storage::FileUtil->make_directory ( $self, $directories, 1 );
   }
+  # any blobs from remote need to exist before we call doc->new
+#XML::Comma::Log->warn("about to set blobs");
+  unless ( $args{blobs_local_files} ) {
+    foreach my $blob ( keys %{$args{blobs}} ) {
+      next unless $blob; #TODO: trace down why this is happening...
+#XML::Comma::Log->warn("setting blob: $blob");
+      open (my $fh, ">$blob") || die "TODO: proper error message: $@";
+      print $fh $args{blobs}->{$blob};
+      close($fh) || die "TODO: proper error message: $@";
+    }
+  }
+#XML::Comma::Log->warn("done setting blobs");
   # make the doc
-  my $doc = XML::Comma::Doc->new ( block => $args{doc_string},
-                                   no_read_hooks => 1 );
+  my $doc = XML::Comma::Doc->new ( block => $args{doc_string}, no_read_hooks => 1 );
   # set storage info
   $doc->set_storage_info ( $self, $location, $args{id}, $args{key} );
   # write
@@ -218,12 +239,20 @@ sub force_store {
                  no_blobs => 1,
                  keep_open => 1 );
   # and walk through the blobs, clearing and resetting according to
-  # the passed list
+  # either our local references or the passed list
   foreach my $blob ( $doc->get_all_blobs() ) {
-    my $send_side_filename = $blob->get_location();
-    $blob->set();
-    $blob->set ( $args{blobs}->{$send_side_filename},
-                 filename => $send_side_filename );
+    if ( $args{blobs_local_files} ) {
+      my $filename = $blob->get_location;
+      $blob->clear_location;
+      print "resetting and copying: " . $filename . "\n";
+      $blob->set_from_file ( $filename );
+    } else {
+#			#this should already have been done by first blob loop
+#			my $send_side_filename = $blob->get_location();
+#			$blob->set();
+#			$blob->set ( $args{blobs}->{$send_side_filename},
+#									 filename => $send_side_filename );
+    }
   }
   $doc->store ( no_hooks => $args{no_hooks} );
   return $doc;
@@ -422,6 +451,20 @@ sub dir_permissions {
   return $_[0]->{_Store_dir_permissions};
 }
 
+sub def_name {
+  return $_[0]->{_Store_doctype};
+}
+
+#this just calls associated_indices for people who spell different
+sub associated_indexes {
+  my ($self, @args) = @_;
+  $self->associated_indices(@args);
+}
+
+sub associated_indices {
+  my $ios = $_[0]->{_index_on_stores};
+  return (defined($ios) && @$ios ? @$ios : ());
+}
 
 sub init_and_cast {
   my ( $self, $document_type ) = @_;
